<?php
/*
  $Id: view_counter, v 1.0 2012/07/01 by Jack York

  osCommerce, Open Source E-Commerce Solutions
  http://www.oscommerce.com

  Copyright (c) 2003 osCommerce
  Portions Copyright 2012 oscommerce-solution.com

  Released under the GNU General Public License
*/
     
/*******************************************************************************
Add the given IP to the .htaccess file and the banned IP DB table
*******************************************************************************/
function AlterHtaccessFile($ip, $isDomain = false) {
    if (strpos($ip, "/") === FALSE && ! filter_var($ip, FILTER_VALIDATE_IP, ['flags' => FILTER_FLAG_IPV4]) && ! $isDomain) { //don't check CIDR's
        return false;
    }
    $dot = '';
    
    if ($isDomain) {                  //already verified to be a domain name
        $dot = '.';                   //needed for a domain name to block all entries
    } elseif (! IsValidIP($ip)) {    //this is an IP or Range so check it
        return false;
    }
   
    $arrayWrite = [];
    $htaccessfile = '';
    $arrayWrite = GetHtaccessFileContents($htaccessfile);
    
    if ($arrayWrite) {    
        $endLine = '';
 
        $arrayWrite = CleanHtaccess($arrayWrite);  //remove any duplicates that might have crept in   
        $items = count($arrayWrite);

        for ($i = 0; $i < $items; ++$i) {
     
            //if (preg_match('/\b' . $ip . '\b/', $arrayWrite[$i])) {
            if (strpos($arrayWrite[$i], $ip) !== false) {
                return false;
            }
            
            if (strpos(addslashes($arrayWrite[$i]), '# End IP deny by View Counter from www.oscommerce-solution.com') !== false) {
                $endLine = $i;
                break;
            }                 
        }
        
        reset($arrayWrite);
        
        if ($endLine) {  //htaccess already has View Counter changes so add to them
            $arrayTmp = [];
            $arrayTmp[] = 'deny from ' . $dot . $ip . PHP_EOL;
            array_splice($arrayWrite, $endLine - 1, 0, $arrayTmp);
        } else {        
            $arrayWrite[] = '# Begin IP deny by View Counter from www.oscommerce-solution.com' . PHP_EOL;      
            $arrayWrite[] = 'order allow,deny' . PHP_EOL;
            $arrayWrite[] = 'deny from ' . $dot . $ip . PHP_EOL;
            $arrayWrite[] = 'allow from all' . PHP_EOL;
            $arrayWrite[] = '# End IP deny by View Counter from www.oscommerce-solution.com' . PHP_EOL;
        }
    } else { //create a new file
        $arrayWrite[] = '# Begin IP deny by View Counter from www.oscommerce-solution.com' . PHP_EOL;      
        $arrayWrite[] = 'order allow,deny' . PHP_EOL;
        $arrayWrite[] = 'deny from ' . $dot . $ip . PHP_EOL;
        $arrayWrite[] = 'allow from all' . PHP_EOL;
        $arrayWrite[] = '# End IP deny by View Counter from www.oscommerce-solution.com' . PHP_EOL;
    }
    
    if (! empty($arrayWrite)) {
        if (WriteHTAccess($htaccessfile, $arrayWrite)) {
            IgnoreThisIP($ip, 0, true);     
            DeleteAllIPs($ip);
            return true;
        } else {
            return false; //ERROR_HTACCESS_WRITE_FAILED;
        }
    }  
    return false;    
}  

/*******************************************************************************
Common function to check for hacker code in arg 
*******************************************************************************/
function CheckForHackerCode($view) {
    global $colors;
    
    $cmpArry = GetHackerWords();
                     
    foreach ($cmpArry as $test) {
        if (isset($view['file_name'])) {
            if (preg_match('/' . $test . '/', $view['file_name'])) {
                return ' style="background-color:' . $colors[COLOR_IS_HACKER] . '" '; 
            }
        }
        if (isset($view['arg'])) {
            if (preg_match('/' . $test . '/i', $view['arg'])) {
                return ' style="background-color:' . $colors[COLOR_IS_HACKER] . '" '; 
            }
        }
    }
    if (false) {
    foreach ($cmpArry as $test) {
        if (preg_match('/' . $test . '/', $view['file_name']) || preg_match('/' . $test . '/', $view['arg'])) {
            return ' style="background-color:' . $colors[COLOR_IS_HACKER] . '" '; 
        }
    }
    }
    return '';
}    
    
/*******************************************************************************
Remove duplicate IP's and IP's that are part of a CIDR from the htaccess file 
*******************************************************************************/
function CleanHtaccess($array) {
    $cidrArray = [];
    $clearedArray = [];
    $newArray = [];

    /************** Remove duplicate IP's **************/    
    foreach ($array as $entry) {
        if (($pos = strpos($entry, 'deny from ')) !== FALSE) {  //this is an IP deny entry
            if (strpos($entry, '/') !== FALSE) {                //this is a cidr
                $cidr = trim(substr($entry, $pos + 9));          //get the cidr
                $cidrArray[] = $cidr;            
            }

            if (in_array($entry, $clearedArray)) {
                continue;
            }    
        }
        
        $clearedArray[] = $entry;
    }

    /************** Remove IP's that are part of a CIDR **************/    
    foreach ($clearedArray as $entry) {
        if (($pos = strpos($entry, 'deny from ')) !== false) { //this is an IP deny entry
            if (strpos($entry, '/') === false) {               //this is not a cidr
                $ip = trim(substr($entry, $pos + 9)); 
                foreach ($cidrArray as $cidr) {                 
                    if (IPinCIDR($ip, $cidr)) {
                        continue;
                    }    
                }
            }
        } 
        $newArray[] = $entry; 
    }
    
    return $newArray;
}

/*******************************************************************************
Convert a range into a CIDR or just use the given IP or CIDR 
*******************************************************************************/
function ConvertRangeToCIDR($ips) {
    $ipArray = [];
    
    if (strpos($ips, '-') !== false) { //this is a range
        $parts = explode('-',$ips);
        $ipArray = IPRangeToCIDR($parts);
    } else {
        $ipArray[] = $ips;
    }
    
    return $ipArray;
}

/*******************************************************************************
Delete all of the IP's in the given CIDR
*******************************************************************************/
function DeleteAllIPs($CIDR) {
    if (strpos($CIDR, '/') !== false) {
        $sqlStr = 'delete from view_counter where ( ';
        
        $ips_query = tep_db_query("select distinct INET_NTOA(ip_number) as ip from view_counter");
        $match = false;
        
        while ($ips = tep_db_fetch_array($ips_query)) {
            if (IPinCIDR($ips['ip'], $CIDR) == 1) {
                $sqlStr .= "ip_number = INET_ATON('" . $ips['ip'] . "') or ";
                $match = true;
            }
        }
        
        if ($match) {
            $sqlStr = substr($sqlStr, 0,-3);
            $sqlStr .= ' )';

            tep_db_query($sqlStr);
            $sqlStr = str_replace('view_counter', 'view_counter_flags', $sqlStr);
            tep_db_query($sqlStr);
        } 
    } else {
        DeleteThisIP($CIDR);
    }    
}

/*******************************************************************************
Delete the given IP from the IP table and Flag entry
*******************************************************************************/
function DeleteThisIP($ip) {
    tep_db_query("delete from view_counter where ip_number = INET_ATON('" . $ip . "')");
    tep_db_query("delete from view_counter_flags where ip_number = INET_ATON('" . $ip . "')");
}

/*******************************************************************************
Format the refresh line and return
*******************************************************************************/
function DisplayRefreshTime() {
    $time = time(); 
    $last_time = @date('H:i:s',$time);  
     
    $next = sprintf("+%d seconds", VIEW_COUNTER_PAGE_REFRESH); 
    //$last_time_plus = date('H:i:s',strtotime($next,$time)); 

    if ((int)VIEW_COUNTER_PAGE_REFRESH > 0) {
        $last_time_plus = '<b id="timeto_refresh">'. VIEW_COUNTER_PAGE_REFRESH . '</b>';
        return sprintf(TEXT_LAST_REFRESH, $last_time, $last_time_plus);
    }    
    
    return sprintf(TEXT_LAST_REFRESH_NO_REFRESH, $last_time);
}

/*******************************************************************************
Format the column heading with the sort indicator
*******************************************************************************/
function DisplayHeading_With_Sort($col_name, $sort_indicator) {
    return sprintf('<span class="monitor-heading">' . $col_name . ' ' . $sort_indicator);
}
    
/*******************************************************************************
Get the number of visits in the last 24 hours
*******************************************************************************/
function GetAccessCount($language_id) {
    $count = '';
    $accessedCnt_query = tep_db_query("SELECT count(*) as count FROM view_counter WHERE last_date >= now() - INTERVAL 1 DAY and language_id = '" . (int)$language_id . "'");
    
    if (tep_db_num_rows($accessedCnt_query)) {
        $accessedCnt = tep_db_fetch_array($accessedCnt_query);
        $count = sprintf(TEXT_ACCESSS_COUNT, $accessedCnt['count']);
    }

    return $count;    
}
  
/*******************************************************************************
Get all of the stats
*******************************************************************************/
function GetAccessCountAll() {
    $countArray = [];
    $accessedCnt_query = tep_db_query("SELECT distinct * FROM view_counter_storage");
    
    if (tep_db_num_rows($accessedCnt_query)) {
        $admin  = 0;
        $bots   = 0;
        $mobile = 0;
        while ($accessedCnt = tep_db_fetch_array($accessedCnt_query)) {
            $admin  = ($accessedCnt['isadmin'] ? $admin + 1 : $admin);
            $bots   = ($accessedCnt['isbot'] ? $bots + 1 : $bots);
            $mobile = (IsMobileDevice($accessedCnt['user_agent']) ? $mobile + 1 : $mobile);
        }
        
        $countArray['total']    = tep_db_num_rows($accessedCnt_query);
        $countArray['admin']  = $admin;
        $countArray['bots']   = $bots;
        $countArray['desktop'] = (tep_db_num_rows($accessedCnt_query) - $mobile);
        $countArray['mobile'] = $mobile;
    }    
    return $countArray;    
}
       
/*******************************************************************************
Get the correct color for the list
*******************************************************************************/
function GetColor($view) {
    global $bannedList, $bannedListLong, $colors;
    
    $colorArray = ['args' => '', 'visitor' => ''];

    if (VC_in_multi_array($view['ip_number_long'], $bannedListLong)) {
        $colorArray['FG'] = 'style="color:' . $colors[COLOR_BUTTONS] . ';"';
    }    

    if ($view['isbot']) {
        $colorArray['BG'] = ' style="background-color:' . $colors[COLOR_IS_BOT] . ';" ';
    } elseif (tep_not_null($view['isadmin'])) {
        $colorArray['BG'] = ' style="background-color:' . $colors[COLOR_IS_ADMIN] . ';"';
    } else {
        $colorArray['BG'] = '';
    }

    $isHacker = CheckForHackerCode($view);
    if (isset($isHacker[0])) {
        $colorArray['args'] = $isHacker;
        $colorArray['file_name'] = $isHacker;
    } else {
        $colorArray['file_name'] = ($view['file_name'] == 'view_counter_trap_badbot.php' ? ' style="background-color:' . $colors[COLOR_IS_BOT_TRAP] . ';"' : '');
    }
    
    return $colorArray;
} 

/*******************************************************************************
Get an array of countries
*******************************************************************************/
function GetCountriesList() {
    $cntryArray = [];
    $cntryList = 'includes/view_counter_countries_list.txt';
    $fp = file($cntryList);  
    $cntryArray = [];
    $cntryArray[] = array('id' => TEXT_BLOCK_COUNTRIES_SELECT, 'text' => TEXT_BLOCK_COUNTRIES_SELECT);

    foreach ($fp as $country) {
        $country = trim($country);
        $cntryArray[] = array('id' => $country, 'text' => $country);
    }  
    return $cntryArray;
}             

/*******************************************************************************
Common function to build an array of possible hacking words. This array is also
found on the shop side so change it when this is changed.
*******************************************************************************/
function GetHackerWords() {
    return array("(\<|%3C)script(>|%3E)", "base64_encode", "(\<|%3C).*iframe", "(\<|%3C)(.*)(\>|%3E)", "javascript:",
                 "eval\(\$_", "filesmanager", "javascript:", "mysql_query", "..\/cmd", "phpinfo\(", "\/iframe", "\$_GET",
                 "\$_POST", "\$_SESSION", "\$_REQUEST", "\$_SERVER", "\$_ENV", "GLOBALS\[", "\$HTTP_", "OR 1=1",
                 "autoLoadConfig", "php\/login", "alert\(", "convert\(", "mysite\.com",
                 "misc.php", "cltreq.asp", "owssvr.dll", "cmd.exe", "awstats", "phpmyadmin", "openwebmail", "formmail",
                 "drop table", "from table", "IDSAlert", "exec", "link_connect", "xp_dirtree", "action=select",
                 "action=%2fshopping_cart.php", "action=%5cshopping_cart.php", "action=shopping_cart.php"  
                 ); 
}
 
/*******************************************************************************
Get domain details for the given IP
*******************************************************************************/
function GetIPDetails($ipAddr) {
    //This site or product includes IP2Location LITE data available from http://lite.ip2location.com 
    //get the country name from local DB
  
    //verify the IP address for the
    if (ip2long($ipAddr)== -1 || ip2long($ipAddr) === false) return '';
 
    require_once(DIR_FS_CATALOG . 'view_counter/IP2Location.php'); 
    $db = new \IP2Location\Database(DIR_FS_CATALOG . 'view_counter/' . VIEW_COUNTER_DATABASE, \IP2Location\Database::FILE_IO);

   // $records = $db->lookup('2603:9009:b10:7d00:fdf9:24c7:bcf5:dacd', \IP2Location\Database::ALL);
    $records = $db->lookup($ipAddr, \IP2Location\Database::ALL);

    if (tep_not_null($records)) {
        $arry = [];
        $arry['IP Address'] = $records['ipAddress'];
        $arry['IP Number'] = $records['ipNumber'];
        $arry['Country Code'] = $records['countryCode'];
        $arry['Country Name'] = $records['countryName'];
        
        return $arry; 
    }
    
    if (VIEW_COUNTER_COUNTRIES_CHECK_ADMIN == 'External') {
     
        //get the country name from geoplugin.net
        $ip_data = @json_decode(file_get_contents("http://www.geoplugin.net/json.gp?ip=".$ipAddr));
    
        if($ip_data && $ip_data->geoplugin_countryName != null) {
         
            $entriesArray = GetEntriesArray(1);
            $geoKeyArray  = array(0 => 'geoplugin_request',1 => 'geoplugin_countryCode',2 => 'geoplugin_countryName',3 => 'geoplugin_regionName',4 => 'geoplugin_areaCode',5 => 'geoplugin_city',6 => 'geoplugin_latitude',7 => 'geoplugin_longitude'); 
 
            $newArray = []; 
            $cnt = count($geoKeyArray);      
            $ip_data = (array)$ip_data;          

            foreach ($ip_data as $entry => $data) {   
                for ($i = 0; $i < $cnt; ++$i) {
                    if (isset($entry[1]) && $entry == $geoKeyArray[$i]) {
                        $newArray[$entriesArray[$i]] = $data;
                        break;
                    }                 
                } 
            } 
            
            return $newArray;         
        }
 
        //get the country name from hostip.info
        // This notice MUST stay intact for legal use            
        if (function_exists('curl_exec')) {
            $curl = curl_init();
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_URL, "http://api.hostip.info/?ip=".$ipAddr);
            $xml = curl_exec($curl);
            
            if (curl_errno($curl)) {
                $xml = @file_get_contents("http://api.hostip.info/?ip=".$ipAddr);  //may be disabled on the server
            }                  
            
            curl_close($curl);
        } else {
            $xml = @file_get_contents("http://api.hostip.info/?ip=".$ipAddr);  //may be disabled on the server
        }

        $entriesArray = GetEntriesArray(2);               
        $hostKeyArray = array(0 => 'ip',1 => 'countryName',2 => 'countryAbbrev');
        $newArray = [];
        $cnt = count($hostKeyArray);
        
        for ($i = 0; $i < $cnt; ++$i) {
            $str = "@<" . $hostKeyArray[$i] . ">(.*?)</" . $hostKeyArray[$i] . ">@si";
            preg_match($str,$xml,$matches);
            $newArray[$entriesArray[$i]] = $matches[1];
        } 

        return $newArray;     
    }
    
    return '';
}

/*******************************************************************************
Get the stored flag data for this IP - used instead of cookies
*******************************************************************************/
function GetDBCookie($ip) {
    $db_query = tep_db_query("select data from view_counter_flags where ip_number = INET_ATON('" . tep_db_input($ip) . "')");
    if (tep_db_num_rows($db_query)) {
        $db = tep_db_fetch_array($db_query);
        return $db['data'];
    }
    return '';
}
    
function GetDomainLocation($ip) {
    $visitorGeolocation = [];
    $storedData = GetDBCookie($ip);
      
    if (! $storedData) {
        $visitorGeolocation = GetIPDetails($ip);
        
        if (is_array($visitorGeolocation) && ! isset($visitorGeolocation['countryCode'])) {
            $visitorGeolocation['countryCode'] = '';
        }
 
        if (isset($visitorGeolocation['countryCode']) && tep_not_null($visitorGeolocation['countryCode']) ) {
            $data = base64_encode(serialize($visitorGeolocation));
            tep_db_query("insert into view_counter_flags (ip_number, data) values (inet_aton('" . $ip . "'), '" . $data . "')");
        }
    } else {
        $visitorGeolocation = unserialize(base64_decode($storedData));
    }
    
    return $visitorGeolocation;
}

/*******************************************************************************
Common function to make managing the possible entries easier
*******************************************************************************/
function GetEntriesArray($type = 0) {
    switch ($type) {
        case 0: $entriesArray = array('IP Address','Country Code','Country Name','Region Name','Areacode','City Name','latitude',
                 'Longitude','IP Number','zipCode','timeZone','isp','domainName','netSpeed','iddCode',
                 'weatherStationCode','weatherStationName','mcc','mnc','mobileCarrierName', 'elevation', 'usageType');
        break;
    
        case 1: $entriesArray = array(0 => 'IP Address',1 => 'Country Code',2 => 'Country Name',3 => 'Region Name',4 => 'Areacode',5 => 'City Name',6 => 'Latitude',7 => 'Longitude');
        break;
        
        //hostip
        case 2: $entriesArray = array(0 => 'IP Address',1 => 'Country Name',2 => 'Country Code');
        break;
    
        default: $entriesArray = array('ipAddress','ipNumber','countryCode','countryName','regionName','cityName','latitude',
                 'longitude','zipCode','timeZone','isp','domainName','netSpeed','iddCode','areaCode',
                 'weatherStationCode','weatherStationName','mcc','mnc','mobileCarrierName', 'elevation', 'usageType');
    }           
                             
    return $entriesArray;             
}                             

/*******************************************************************************
Build a list of IP's to ignore. They are taken from the Ignore list in the 
database and the ignore admin IP setting.
*******************************************************************************/
function GetHiddenIPs($ignoreList) {
    $hideIPs = '';
    for ($l = 1; $l < count($ignoreList); ++$l) { //add the ignore IP's
        $hideIPs .= "'" . $ignoreList[$l]['text'] . "',";       
    }
    if (tep_not_null(VIEW_COUNTER_HIDE_ADMIN_LINKS)) { //add the IP's in the admin setting
        $hideIPs .= "'" . str_replace(',', "','", str_replace(' ', '', trim(VIEW_COUNTER_HIDE_ADMIN_LINKS, ','))) . "'";         //condition the string from the setting
    }
    if (substr($hideIPs, -1) == ",") {     //remove the last comma, if present
        $hideIPs = substr($hideIPs, 0, -1);
    }    
    
    if (! empty($hideIPs)) {
        $hideIPs = " and INET_NTOA(ip_number) NOT in (" .  $hideIPs . ")";
    } 
    return $hideIPs;
}
/*******************************************************************************
Load the .htaccess file into an array
*******************************************************************************/
function GetHtaccessFileContents(&$htaccessfile = '', $name_only = false) {
    $separator = ((substr(DIR_FS_CATALOG, -1) != '/') ? '/' : '');
    $htaccessfile = DIR_FS_CATALOG . $separator . '.htaccess';

    if ($name_only) {  //the contents are not needed so return
        return true;
    }

    if (file_exists($htaccessfile)) {
        $htaccessArray = file($htaccessfile);
        return $htaccessArray;
    }
    return false;
}

/*******************************************************************************
Returns a comma separated string used in a DB call to prevent IP's from
showing if they are in a CIDR
*******************************************************************************/
function GetInCIDRStr($htaccessfile, $storage_table, $language_id, $show_only_this_ip, $showType, $hideAdminLinks, $sortBy, $whereActive,  $whereSpoofed, $whereHackers) {
    $view_query = tep_db_query("select vc.ip_number as ip from " . $storage_table . " vc where vc.ip_number NOT in (select vcb.ip_number from view_counter_banned vcb where ignore_status <> 1 ) and ip_active=1 and language_id = " . (int)$language_id . $show_only_this_ip . $showType . $whereActive . $whereSpoofed . $whereHackers . $hideAdminLinks . $sortBy . " ");
    $inCIDRStr = '';
    $inCIDRArray = [];
    $ctr = 0;
    
    while ($view = tep_db_fetch_array($view_query)) {
        if (! isset($inCIDRArray[$view['ip']])) {
            $check = false;
            if (IsIPinCIDR($view['ip'], $htaccessfile, $check) && $check) {
                $inCIDRArray[$view['ip']] = $ctr++;
            }
        }
    }

    if (count($inCIDRArray) > 1) {
        $inCIDRArray = array_flip($inCIDRArray);
        $inCIDRStr = implode("','", $inCIDRArray);
        $inCIDRStr = " and ( INET_NTOA(vc.ip_number) NOT IN  ( '" . $inCIDRStr . "' ))";
    }

    return $inCIDRStr;
}

function GetInCIDRStrOrig($htaccessfile, $storage_table, $language_id, $show_only_this_ip, $showType, $hideAdminLinks, $sortBy, $whereActive,  $whereSpoofed) {
    $view_query = tep_db_query("select vc.ip_number from " . $storage_table . " vc where vc.ip_number NOT in (select vcb.ip_number from view_counter_banned vcb where ignore_status <> 1 ) and ip_active=1 and language_id = " . (int)$language_id . $show_only_this_ip . $showType . $whereActive . $whereSpoofed . $hideAdminLinks . $sortBy . " ");
    $inCIDRStr = '';
    $inCIDRArray = [];

    while ($view = tep_db_fetch_array($view_query)) {
        if (! in_array($view['ip'], $inCIDRArray)) {
            $check = false;
            if (IsIPinCIDR($view['ip'], $htaccessfile, $check) && $check) {
                $inCIDRArray[] = $view['ip'];
            }
        }
    }

    if (count($inCIDRArray) > 1) {
        $inCIDRStr = implode("','", $inCIDRArray);
        $inCIDRStr = " and ( INET_NTOA(vc.ip_number) NOT IN  ( '" . $inCIDRStr . "' ))";
    }

    return $inCIDRStr;
}

/*******************************************************************************
Returns the longest str
*******************************************************************************/
function GetLongestStr($pName, $longest) {
    $nameLength = strlen($pName); //get the longest name for better appearance

    if ($nameLength > $longest) {
        $longest = $nameLength;
    }

    return $longest;
}

/*******************************************************************************
Common function to populate the passed array with the fields to prevent php warnings
*******************************************************************************/
function GetSortByArrayFields(&$sortArray = []) {
 
    $keys = [TEXT_SORT_BY_ARG, 
             TEXT_SORT_BY_COUNT,
             TEXT_SORT_BY_BANNED_FROM_WHERE, 
             TEXT_SORT_BY_BANNED_FROM_IP, 
             TEXT_SORT_BY_BANNED_FROM_TYPE, 
             TEXT_SORT_BY_COUNTRY, 
             TEXT_SORT_BY_FLAG, 
             TEXT_SORT_BY_DATE, 
             TEXT_SORT_BY_FILENAME, 
             TEXT_SORT_BY_IP, 
             TEXT_SORT_BY_ISBOT, 
             TEXT_SORT_BY_VISITOR];

    if (! is_array($sortArray)) $sortArray = [];
    
    foreach ($keys as $key) {
        if (!array_key_exists($key, $sortArray)) {
            $sortArray[$key] = '';
        }
    }
    return $sortArray;
}
    
/*******************************************************************************
Get the sorting method and direction based on the column clicked
*******************************************************************************/
function GetSorting(&$sortByArray, &$sortBy, $cond, &$sortDirection, $report_name = null, $override_field = null) {
    $direction = ' asc ';
    $sortByArray = [];

    //Set the column and direction
    switch ($cond) {
        case TEXT_SORT_BY_ARG: 
            $direction = ((strpos($sortBy, TEXT_SORT_BY_ARG) !== false) ?  ((strpos($sortBy, 'asc') !== false) ? ' desc ' : ' asc ') : ' asc ');
            $sortByArray[TEXT_SORT_BY_ARG] = 'checked'; $sortBy = ' order by arg ' . $direction; 
        break;

        case TEXT_SORT_BY_COUNT: 
            $direction = ((strpos($sortBy, TEXT_SORT_BY_COUNT) !== false) ?  ((strpos($sortBy, 'asc') !== false) ? ' desc ' : ' asc ') : ' asc ');
            $sortByArray[TEXT_SORT_BY_COUNT] = 'checked'; $sortBy = ' order by view_count ' . $direction; 
        break;
        
        case TEXT_SORT_BY_COUNTRY: 
            $direction = ((strpos($sortBy, TEXT_SORT_BY_COUNTRY) !== false) ?  ((strpos($sortBy, 'asc') !== false) ? ' desc ' : ' asc ') : ' asc ');
            $sortByArray[TEXT_SORT_BY_COUNTRY] = 'checked'; $sortBy = ' order by country ' . $direction; 
        break;
        
        case TEXT_SORT_BY_DATE: 
            $direction = ((strpos($sortBy, TEXT_SORT_BY_DATE) !== false) ?  ((strpos($sortBy, 'asc') !== false) ? ' desc ' : ' asc ') : ' asc ');
            $sortByArray[TEXT_SORT_BY_DATE] = 'checked'; $sortBy = ' order by last_date ' . $direction; 
        break;
        
        case TEXT_SORT_BY_FILENAME: 
            $direction = ((strpos($sortBy, TEXT_SORT_BY_FILENAME) !== false) ?  ((strpos($sortBy, 'asc') !== false) ? ' desc ' : ' asc ') : ' asc ');
            $sortByArray[TEXT_SORT_BY_FILENAME] = 'checked'; $sortBy = ' order by file_name ' . $direction; 
        break;
        
        case TEXT_SORT_BY_IP: 
            $direction = ((strpos($sortBy, TEXT_SORT_BY_IP) !== false) ?  ((strpos($sortBy, 'asc') !== false) ? ' desc ' : ' asc ') : ' asc '); 
            $sortByArray[TEXT_SORT_BY_IP] = 'checked'; $sortBy = ' order by ip_number ' . $direction; 
        break;
        
        case TEXT_SORT_BY_ISBOT: 
            $direction = ((strpos($sortBy, TEXT_SORT_BY_ISBOT) !== false) ?  ((strpos($sortBy, 'asc') !== false) ? ' desc ' : ' asc ') : ' asc ');
            $sortByArray[TEXT_SORT_BY_ISBOT] = 'checked'; $sortBy = ' order by isbot ' . $direction; 
        break;
        
        case TEXT_SORT_BY_VISITOR: 
            $direction = ((strpos($sortBy, TEXT_SORT_BY_VISITOR) !== false) ?  ((strpos($sortBy, 'asc') !== false) ? ' desc ' : ' asc ') : ' asc ');
            $sortByArray[TEXT_SORT_BY_VISITOR] = 'checked'; $sortBy = ' order by bot_name ' . $direction;
        break;
        
        case TEXT_SORT_BY_BANNED_FROM_IP: 
            $direction = ((strpos($sortBy, TEXT_SORT_BY_BANNED_FROM_IP) !== false) ?  ((strpos($sortBy, 'asc') !== false) ? ' desc ' : ' asc ') : ' asc '); 
            $sortByArray[TEXT_SORT_BY_BANNED_FROM_IP] = 'checked'; $sortBy = ' order by from_ip ' . $direction; 
        break;
        
        case TEXT_SORT_BY_BANNED_FROM_WHERE: 
            $direction = ((strpos($sortBy, TEXT_SORT_BY_BANNED_FROM_WHERE) !== false) ?  ((strpos($sortBy, 'asc') !== false) ? ' desc ' : ' asc ') : ' asc '); 
            $sortByArray[TEXT_SORT_BY_BANNED_FROM_WHERE] = 'checked'; $sortBy = ' order by from_where ' . $direction; 
        break;
        
        case TEXT_SORT_BY_BANNED_FROM_TYPE: 
            $direction = ((strpos($sortBy, TEXT_SORT_BY_BANNED_FROM_TYPE) !== false) ?  ((strpos($sortBy, 'asc') !== false) ? ' desc ' : ' asc ') : ' asc '); 
            $sortByArray[TEXT_SORT_BY_BANNED_FROM_TYPE] = 'checked'; $sortBy = ' order by banned_type ' . $direction; 
        break;

        default: 
            $direction = ((strpos($sortBy, 'asc') !== false) ? ' desc ' : ' asc '); 
            $sortByArray[TEXT_SORT_BY_DATE] = 'checked'; $sortBy = ' order by last_date ' . $direction; 
        break;
    }

    $sortDirection[$cond] = (strpos($direction, 'desc') !== false ? '<i class="fas fa-long-arrow-alt-down sort-arrow"></i>' : 
                                                                    '<i class="fas fa-long-arrow-alt-up sort-arrow"></i>');
}

/*******************************************************************************
Get the color of the column background for mouseovers
*******************************************************************************/
function GetSortingColumColor($sortByArray, $colors, $sortDirectionArray = []) {
    $menuArray = [];
    $menuNames = array(array('name' => 'visitor',   'col' => TEXT_SORT_BY_VISITOR),
                       array('name' => 'file_name', 'col' => TEXT_SORT_BY_FILENAME),
                       array('name' => 'arg',       'col' => TEXT_SORT_BY_ARG),
                       array('name' => 'count',     'col' => TEXT_SORT_BY_COUNT),
                       array('name' => 'ip',        'col' => TEXT_SORT_BY_IP),
                       array('name' => 'date',      'col' => TEXT_SORT_BY_DATE)
                      );
                      
    foreach ($menuNames as $menu) {
        $bkgnd = ($sortByArray[$menu['name']] == 'checked' ? $colors[COLOR_SELECTED_COLUMN] : ''); 
        $menuArray[$menu['name']] = 'onClick="SortColumn(\'' . $menu['col'] . '\',\'view_counter_form\')" style="text-align:center; background-color:' . $bkgnd . '" onmouseover="this.style.background=\'' . $colors[COLOR_SELECTED_COLUMN_HIGHLIGHT] . '\';" onmouseout="this.style.background=\'' . (! empty($sortByArray[$menu['name']]) ? $colors[COLOR_SELECTED_COLUMN] : '') . '\';" ' ;
    }

    return $menuArray;
}                            

/*******************************************************************************
Set the sorting method to $default in reports when none is set
*******************************************************************************/
function GetSortingDefault(&$sortByArray, &$sortBy, $cond, &$sortDirection, $default, $report_name, $override_field = null) {
    if (empty($sortBy)) {  //methos not selected
       $cond = $default;   //switch to this one
       GetSorting($sortByArray, $sortBy, $cond, $sortDirection, $report_name, $override_field);
    }    
}

/*******************************************************************************
Returns the description of who's online
*******************************************************************************/
function GetVisitorName($bot_name) {
    return (tep_not_null($bot_name) ? $bot_name : TEXT_GUEST);
}

/*******************************************************************************
Create a backup of the .htaccess file
*******************************************************************************/
function HtaccessBackup(&$type) {
    $arrayWrite = [];
    $type = 'error';     

    $separator = ((substr(DIR_FS_CATALOG, -1) != '/') ? '/' : '');
    $backupDir = DIR_FS_CATALOG . $separator . TEXT_HTACCESS_BACKUP_LOCN;
    
    if (! is_dir($backupDir)) {
        if (! mkdir($backupDir, 0755)) {
            return ERROR_HTACCESS_BACKUP_DIR_FAILED;
        }
    }

    $backupDir .= '/';
    $htaccessfileOrig = DIR_FS_CATALOG . $separator . '.htaccess';
    $htaccessfileBkup = $backupDir . '.htaccess_bkup_'. date('Y-m-d');

    $result = copy($htaccessfileOrig, $htaccessfileBkup);

    if ($result) {
        $type = 'success';
        return SUCCESS_HTACCESS_BACKUP;
    } else {
        return ERROR_HTACCESS_WRITE_FAILED;
    }   
} 

/*******************************************************************************
Rebuild the .htaccess file by adding in the View Counter code
*******************************************************************************/
function HtaccessRebuild(&$type) {
    $arrayWrite = [];
    $htaccessfile = '';
    $type = 'error';     
    
    $arrayWrite = GetHtaccessFileContents($htaccessfile);
    
    if ($arrayWrite) {   
    
        $result = HtaccessBackup($type);
        if ($type == 'error') {
            return $result; 
        } else {
            $result = '';
            $type = 'error'; //exit - can't do backup or reset error condition
        }            
        
        $blockArray = [];
        $blockArray[] =  "\n" . '# Begin IP deny by View Counter from www.oscommerce-solution.com' . "\n";
        $blockArray[] = 'order allow,deny' . "\n";
        
        $db_query = tep_db_query("select INET_NTOA(ip_number) as ip_number, ip_string from view_counter_banned where ignore_status = 0");
        if (tep_db_num_rows($db_query) > 0) {
            while ($db = tep_db_fetch_array($db_query)) {
                $ip = ($db['ip_number'] > 0 ? $db['ip_number'] : $db['ip_string']);
                $blockArray[] = 'deny from '. $ip . "\n";
            }
        }
        $blockArray[] = 'allow from all' . "\n";
        $blockArray[] = '# End IP deny by View Counter from www.oscommerce-solution.com';
        
        $arrayWrite = array_merge($arrayWrite, $blockArray);
        $arrayWrite = CleanHtaccess($arrayWrite);
        $result = WriteHTAccess($htaccessfile, $arrayWrite);  

        if ($result) {
            $type = 'success';
            return SUCCESS_HTACCESS_REBUILD;
        } else {
            return ERROR_HTACCESS_WRITE_FAILED;
        }         
    } else {
          return ERROR_HTACCESS_NOT_FOUND;    
    }    
}

/*******************************************************************************
Rebuild the banned list from the .htaccess file
*******************************************************************************/
function HtaccessRebuildBannedList(&$type) {
    $arrayWrite = [];
    $htaccessfile = '';
    $type = 'error';     

    $arrayWrite = GetHtaccessFileContents($htaccessfile);
    
    if ($arrayWrite) { 
        $bannedArray = [];
        $cnt = count($arrayWrite);
        $result = false;
        
        for ($i =0; $i < $cnt; ++$i) {
                
            if (($pos = strpos($arrayWrite[$i], 'deny from')) !== FALSE) {
                $result = true;
                $ip = trim(substr($arrayWrite[$i], $pos + 10));

                if (IsValidIP($ip)) { //it's an ip or a cidr   
                    $db_query = tep_db_query("select 1 from view_counter_banned where ip_number = INET_ATON('" . $ip . "') and ignore_status = 0");
                    if (tep_db_num_rows($db_query)) {
                        continue;
                    }    
                
                    if (strpos($ip, '/') == FALSE) {
                        tep_db_query("insert ignore into view_counter_banned (ip_number, ignore_status) values (INET_ATON('" . $ip . "'), '0')");
                    } else {
                        tep_db_query("insert ignore into view_counter_banned (ip_string, ignore_status) values ('". tep_db_input($ip) . "', '0')");
                    }    
                } else if (IsValidDomainName($ip)) {
                    $db_query = tep_db_query("select 1 from view_counter_banned where ip_string = '" . tep_db_input($ip) . "' and ignore_status = 0");
                    if (tep_db_num_rows($db_query)) {
                        continue;
                    }                      
                    tep_db_query("insert ignore into view_counter_banned (ip_string, ignore_status) values ('". tep_db_input($ip) . "', '0')");
                }
            }                       
        } 
        
        $ip_array = [];
        $db_query = tep_db_query("select distinct ip_number, ip_string, ignore_status from view_counter_banned order by ip_number");
        while ($db = tep_db_fetch_array($db_query)) {
            if ($db['ip_number'] == 0 && empty($db['ip_string'])) { 
                continue;
            } 
            $ip_array[] = $db;
        }
        
        tep_db_query("truncate view_counter_banned");
        foreach ($ip_array as $arr) {
           tep_db_query("insert into view_counter_banned (ip_number, ip_string, ignore_status) values ('" . $arr['ip_number'] . "', '" . $arr['ip_string']. "', '" .$arr['ignore_status'] . "')");
        } 
        
        if ($result) {
            $type = 'success';
            return SUCCESS_HTACCESS_REBUILD_BANNED;
        } else {
            return ERROR_HTACCESS_WRITE_FAILED;
        }         
    }
    return ERROR_HTACCESS_NOT_FOUND;    
}

/*******************************************************************************
Restore the .htaccess file from backup
*******************************************************************************/
function HtaccessRestore($srcfile, &$type) {
    $arrayWrite = [];
    $type = 'error';     

    $separator = ((substr(DIR_FS_CATALOG, -1) != '/') ? '/' : '');
    $backupDir = DIR_FS_CATALOG . $separator . TEXT_HTACCESS_BACKUP_LOCN;    

    $backupDir .= '/';
    $htaccessfileOrig = '';
    GetHtaccessFileContents($htaccessfileOrig, true);
    $htaccessfileBkup = $backupDir . $srcfile;

    if (file_exists($htaccessfileBkup)) {    
        $arrayWrite = file($htaccessfileBkup);  
 
        $result = WriteHTAccess($htaccessfileOrig, $arrayWrite);        
        if ($result) {
            $type = 'success';
            return SUCCESS_HTACCESS_REBUILD;
        } else {
            return ERROR_HTACCESS_WRITE_FAILED;
        }         
    } else {
          return ERROR_HTACCESS_NOT_FOUND;    
    }  
} 

/*******************************************************************************
Whitelist an IP - adds an entry in the .htaccess file to allow access even if
the ip is in a blocked range
*******************************************************************************/
function HtaccessWhitelistIP($ip, &$type) {
    $db_query = tep_db_query("select 1 from view_counter_banned where (ip_number = INET_ATON('" . $ip . "') or ip_string = '" . tep_db_input($ip) . "')");
    
    if (tep_db_num_rows($db_query) > 0) {
        RemoveBannedIP($ip);
        $type = 'success';
        return SUCCESS_HTACCESS_WHITELISTED_IP;
    } 

    $htaccessfile = '';
    $arrayWrite = GetHtaccessFileContents($htaccessfile);
    
    if ($arrayWrite) { 
        $endLine = '';
 
        $arrayWrite = CleanHtaccess($arrayWrite);  //remove any duplicates that might have crept in   
         
        for ($i = 0; $i < count($arrayWrite); ++$i) {
            if (preg_match('/\b' . $ip . '\b/', $arrayWrite[$i])) {
                $type = 'success';
                return SUCCESS_HTACCESS_WHITELISTED_IP;
            }
        }
        
        reset($arrayWrite);

        for ($i = 0; $i < count($arrayWrite); ++$i) {
            if (strpos(addslashes($arrayWrite[$i]), '# End IP allow by View Counter from www.oscommerce-solution.com') !== false) {
                $endLine = $i;
                break;
            }            
        }

        reset($arrayWrite);
        
        if ($endLine) {  //htaccess already has View Counter changes so add to them
            $arrayTmp = [];
            $arrayTmp[] = 'allow from ' . $dot . $ip . PHP_EOL;
            array_splice($arrayWrite, $endLine - 1, 0, $arrayTmp);
        } else {        
            $arrayWrite[] = '# Begin IP allow by View Counter from www.oscommerce-solution.com' . PHP_EOL;      
            $arrayWrite[] = 'order allow,deny' . PHP_EOL;
            $arrayWrite[] = 'allow from ' . $dot . $ip . PHP_EOL;
            $arrayWrite[] = '# End IP allow by View Counter from www.oscommerce-solution.com' . PHP_EOL;
        }
    } else { //create a new file
        $arrayWrite[] = '# Begin IP allow by View Counter from www.oscommerce-solution.com' . PHP_EOL;      
        $arrayWrite[] = 'order allow,deny' . PHP_EOL;
        $arrayWrite[] = 'allow from ' . $dot . $ip . PHP_EOL;
        $arrayWrite[] = '# End IP allow by View Counter from www.oscommerce-solution.com' . PHP_EOL;
    }
    
    if (! empty($arrayWrite)) {
        if (WriteHTAccess($htaccessfile, $arrayWrite)) {
            tep_db_query("update view_counter set ip_active=1 where ip_number = INET_ATON('" . $ip . "')");
            $type = 'success';
            return SUCCESS_HTACCESS_WHITELISTED_IP;
        } else {
            return ERROR_HTACCESS_WRITE_FAILED;
        }  
    }      
}

/*******************************************************************************
Add the given IP as an ignored IP or a banned IP, depending upon the type
*******************************************************************************/
function IgnoreThisIP($ip, $type = STATUS_IGNORE, $skip_log = false) {
    $ip_query = tep_db_query("select ignore_status from view_counter_banned where (ip_number = INET_ATON('" . $ip . "') or ip_string = '" . tep_db_input($ip) . "')");

    if (tep_db_num_rows($ip_query) > 0) { //already being ignored or banned
        $result = tep_db_fetch_array($ip_query);

        if ($result['ignore_status'] == STATUS_BAN && $type == STATUS_BAN) { //already banned
            return ERROR_IP_ALREADY_BANNED;
        } elseif ($result['ignore_status'] == STATUS_IGNORE && $type == STATUS_IGNORE) { //already ignored
            return ERROR_IP_ALREADY_IGNORED;
        } else { //ignore_status doesn't match type so ip is in the other list
            return ($type == STATUS_BAN ? ERROR_IP_BLOCKED_IN_IGNORE : ERROR_IP_BLOCKED_IN_BANNED);
        }
    }
    
    if (! $skip_log) { //already recorded from calling function
        $this_type = TEXT_REPORT_BANNED_LOG_TYPE_BANNED;
        $this_from = TEXT_BLOCK_IP_RANGE;
        if ($type) {
            $this_type = TEXT_REPORT_BANNED_LOG_TYPE_IGNORE;
            $this_from = TEXT_BLOCK_IP_IGNORE;
        }     
        RecordLogChange($ip, $this_type, $this_from);
    }
    
    if (strpos($ip, '/') !== false || IsValidDomainName($ip)) {
        $ips_query = tep_db_query("select distinct INET_NTOA(ip_number) as ip from view_counter");
        $match = false;

        while ($ips = tep_db_fetch_array($ips_query)) {
            if (IPinCIDR($ips['ip'], $ip) == 1) {
                $sqlStr .= " ip_number = INET_ATON('" . $ips['ip'] . "') or ";
                $match = true;
            }
        }
 
        if ($match) {
            $sqlStr = '( ' . substr($sqlStr, 0, -3) . ' )';
            tep_db_query("update view_counter set ip_active=0 where " . $sqlStr);
        }           

        tep_db_query("insert into view_counter_banned (ip_string, ignore_status) values ('" . tep_db_input($ip) . "', " . (int)$type . ")");
    } else {
        tep_db_query("update view_counter set ip_active=0 where ip_number = INET_ATON('" . $ip . "')");
        tep_db_query("insert into view_counter_banned (ip_number, ignore_status) values (INET_ATON('" . $ip . "'), " . (int)$type . ")");
    } 
    return $type; //return status so it can be seen as different from a message
}

/*******************************************************************************
Checks if the given IP is in the CIDR
*******************************************************************************/
function IPinCIDR($ip, $cidr) {
    list ($subnet, $bits) = explode('/', $cidr);
    $ip = ip2long($ip);
    $subnet = ip2long($subnet);
    $mask = -1 << (32 - $bits);
    $subnet &= $mask; 
    return ($ip & $mask) == $subnet;
}

function IPRangeToCIDR($ips) {     
    $resultArray = []; 
     
    $num = ip2long(trim($ips[1])) - ip2long(trim($ips[0])) + 1;     
    $bin = decbin($num);       
    $chunk = str_split($bin);     
    $chunk = array_reverse($chunk);     
    $start = 0;       
    while ($start < count($chunk)) {         
        if ($chunk[$start] != 0) {
            $start_ip = isset($range) ? long2ip(ip2long($range[1]) + 1) : $ips[0];             
            $range = cidr2ip($start_ip . '/' . (32 - $start));             
            $resultArray[] = $start_ip . '/' . (32 - $start);         
        }         
        $start++;     
    }    

    return str_replace(' ', '', $resultArray); 
}

/*******************************************************************************
Returns if a common mobile device was used 
*******************************************************************************/
function IsMobileDevice($useragent) {
    $mobileNames = ['Android', 'iPad', 'iPod', 'iPhone', 'iOS', 'Mobile'];
    
    foreach($mobileNames as $device) {
        if (stripos($useragent, $device) !== FALSE) { 
            return true;
        }
    }    
    return false;    
}
function cidr2ip($cidr) { 
    $ip_arr = explode('/', $cidr);     
    $start = ip2long($ip_arr[0]);     
    $nm = $ip_arr[1];     
    $num = pow(2, 32 - $nm);     
    $end = $start + $num - 1;     
    return array($ip_arr[0], long2ip($end)); 
} 

/*******************************************************************************
Provides every IP in the CIDR. Not currently used. The results can be in the
millions and takes a very long time.
*******************************************************************************/
function cidr2ipFull($cidr) {   
      $ip_addr_cidr = $cidr;
    $ip_arr = explode('/', $ip_addr_cidr);

    $bin = '';
    for($i=1;$i<=32;$i++) {
        $bin .= $ip_arr[1] >= $i ? '1' : '0';
    }
    $ip_arr[1] = bindec($bin);

    $ip = ip2long($ip_arr[0]);
    $nm = ip2long($ip_arr[1]);
    $nw = ($ip & $nm);
    $bc = $nw | (~$nm);

    echo "Number of Hosts:    " . ($bc - $nw - 1) . "<br/>";
    echo "Host Range:         " . long2ip($nw + 1) . " -> " . long2ip($bc - 1) . "<br/>". "<br/>";

    $check = '127.0.0.1';
    $ipa = [];
    for($zm=1;($nw + $zm)<=($bc - 1);$zm++) {
        if (long2ip($nw + $zm) == $check) {
            return true;
        }  
    }
    return false;    
 }

/*******************************************************************************
Checks if the given IP is any of the ranges in the .htaccess file
*******************************************************************************/
function IsIPInCidr($ip, $htaccessArray, &$result) {
    if ($htaccessArray) {
       if ($ip) {
           $offset = strlen('deny from ');
           $items = count($htaccessArray);
           
           for ($i = 0; $i < $items; ++$i) {
               if (strpos($htaccessArray[$i], 'deny') !== FALSE && strpos($htaccessArray[$i], '/') !== FALSE) {
                   $cidr = trim(substr($htaccessArray[$i], $offset));
               
                   if (IPinCIDR($ip, $cidr)) {
                       $result = true;
                       return sprintf(TEXT_IP_IN_CIDR_FOUND, $ip, $cidr);
                   } 
               }
           }

           $result = false;           
           return sprintf(TEXT_IP_IN_CIDR_NOT_FOUND, $ip);
       }    
    }   
    
    $result = false;           
    return ERROR_HTACCESS_NOT_FOUND;
}

/*******************************************************************************
Checks for a valid domain name - assumes no more than two dots and the rest
alphnumeric
*******************************************************************************/
function IsValidDomainName($domain) {
    if ($domain[0] == '.') {
        $domain = substr($domain, 1); //remove the leading 0 if present
    }   
    if (! ctype_alpha($domain[0])) { //not an IP
        return false;
    }    
    
    return (! filter_var( $domain, FILTER_VALIDATE_DOMAIN ) ? false : true);   
}

/*******************************************************************************
Checks for a valid IP address or optionally a cidr notation range
e.g. 1.2.3.4 or 1.2.3.0/24
*******************************************************************************/
function IsValidIP($cidr) {
    if ($cidr == '0.0.0.0') return false;
    
    if (filter_var($cidr, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)) return true;
   
    if (strpos($cidr, '/') !== false) {
        list ($net, $mask) = explode("/", $cidr);
        if (! filter_var($net, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)) return false;
        if (ip2long($net) == false) return false;;
        if ($mask > 32) return false;
    } elseif (! ctype_digit($ip)) {     
        return false;
    } 
    
    return true;
}

/*******************************************************************************
Kick the given IP off of the shop
*******************************************************************************/
function KickOffIP($ip, $session, $msg = '') {
    tep_db_query("insert into view_counter_session_kill (session_id, session_msg) values ('" . tep_db_input($session) . "', '" . tep_db_input($msg) . "')");     
}

/*******************************************************************************
Load just the color settings and the selected preset mode 
*******************************************************************************/
function LoadColorSettings(&$colors) {
    $mode = COLOR_PRESET_DEFAULT;
    $settings_query = tep_db_query("select * from view_counter_settings");

    if (tep_db_num_rows($settings_query)) {
        $settings = tep_db_fetch_array($settings_query);
        $color = [];
        $colors = unserialize( base64_decode( $settings['colors'] ) );        
        $mode = $colors['mode'];
        unset($colors['mode']);
    }
    
    return $mode;
}

/*******************************************************************************
Load the last settings - full set - only called in view_counter.php (not reports)
*******************************************************************************/
function LoadSettings(&$show_only_active, &$show_only_spoofed, &$show_only_hackers, &$showType, &$showTypeArray, &$sortBy, &$sortByArray, &$sortDirectionArray, &$display_table, &$colors, $page_name = 'monitor') {
    $settings_query = tep_db_query("select * from view_counter_settings where page_name = '" . tep_db_input($page_name) . "'");

    if (tep_db_num_rows($settings_query)) {     
        $settings = tep_db_fetch_array($settings_query);

        $show_only_active = $settings['view_active'];
        $show_only_spoofed = $settings['view_spoofed'];
        $show_only_hackers = $settings['view_hackers'];
        $showType = $settings['show_type'];
        $showTypeArray = unserialize( base64_decode( $settings['show_type_array'] ) );
        $sortBy = $settings['sort_by'];
        $sortByArray = unserialize( base64_decode( $settings['sort_by_array'] ) );  
        $sortDirectionArray = unserialize( base64_decode( $settings['sort_direction_array'] ) );  
    
        GetSortByArrayFields($sortDirectionArray); //add missing fields
        GetSortByArrayFields($sortByArray);

        $display_table = $settings['display_table'];
        $colors = unserialize( base64_decode( $settings['colors'] ) );   
    }
}

/*******************************************************************************
Load the last settings - reports only
*******************************************************************************/
function LoadSettingsReports($page_name, &$group_data) {
    $settings_query = tep_db_query("select group_data from view_counter_settings where page_name = '" . tep_db_input($page_name) . "'");

    if (tep_db_num_rows($settings_query)) {     
        $settings = tep_db_fetch_array($settings_query);
        $group_data = unserialize( base64_decode( $settings['group_data'] ) );
        return true;
    }
    
    return false;
}

/*******************************************************************************
Remove the given IP from the the banned DB table and the .htaccess file
*******************************************************************************/
function RemoveBannedIP($ip) {
    $arrayWrite = [];
    $htaccessfile = '';
    $arrayWrite = GetHtaccessFileContents($htaccessfile);
    
    if ($arrayWrite) { 
        $endLine = '';
        
        $dot = (IsValidDomainName($ip) ? '.' : '');
        
        for ($i = 0; $i < count($arrayWrite); ++$i) {
            if (strpos($arrayWrite[$i], 'deny from ' .$dot . $ip) !== false) {
                unset($arrayWrite[$i]);
            }            
        }
        
        reset($arrayWrite);
                
        if (! WriteHTAccess($htaccessfile, $arrayWrite)) {
            return false;
        }
    }    

    RemoveIgnoredIP($ip);
    return true;
}


/*******************************************************************************
Remove the given IP or Domain name from the the banned DB table and the .htaccess 
file.
*******************************************************************************/
function RemoveIgnoredIP($ip) {
    if (strpos($ip, '/') !== FALSE || IsValidDomainName($ip)) {
        $ips_query = tep_db_query("select distinct INET_NTOA(ip_number) as ip from view_counter");
        $match = false;

        while ($ips = tep_db_fetch_array($ips_query)) {
            if (IPinCIDR($ips['ip'], $ip) == 1) {
                $sqlStr .= " ip_number = INET_ATON('" . $ips['ip'] . "') or ";
                $match = true;
            }
        }

        if ($match) {
            $sqlStr = '( ' . substr($sqlStr, 0, -3) . ' )';
            tep_db_query("update view_counter set ip_active=1 where " . $sqlStr);
        }
        tep_db_query("delete from view_counter_banned where ip_string = '" . tep_db_input($ip) . "'");
    } else {
        tep_db_query("update view_counter set ip_active=1 where ip_number = INET_ATON('" . $ip . "')");
        tep_db_query("delete from view_counter_banned where ip_number = INET_ATON('" . $ip . "')");
    } 
}

/*******************************************************************************
Save the current settings 
*******************************************************************************/
function SaveColorSettings($post) {

    $mode = COLOR_PRESET_DEFAULT;
    foreach ($post as $id => $color) {
        if ($id == 'update' || $id == 'color_preset' || $id == 'osCAdminID') { //remove known non-color posts
            if ($id == 'color_preset') {
                $mode = $color;     //save the selected mode
            }
            unset ($post[$id]);
        } else if (strpos($color, '#') === FALSE) {
            return false;
        }
    }

    $post['mode'] = $mode;
    $colors = base64_encode( serialize( $post) );  
    $check_query = tep_db_query("select 1 from view_counter_settings");    

    if (tep_db_num_rows($check_query) > 0) {
        tep_db_query("update view_counter_settings set colors = '" . tep_db_input($colors) . "' where page_name='monitor'"); 
    } else {    
        $sortByArray = array('date' => '', 'file_name' => 'checked', 'ip' => '', 'count' => '');
        $sortBy = ' order by file_name asc ';
        $sort_direction_array = '';
        $showTypeArray = array('bots' => '', 'customers' => 'checked', 'both' => '');
        $showType = ' and isbot = 0 ';
        $show_only_active = '';
        $show_only_spoofed = '';
        $show_only_hackers = '';
        $display_table = '';
        $group_data = '';

        $stringShow = base64_encode( serialize( $showTypeArray ) );  
        $stringSort = base64_encode( serialize( $sortByArray ) );         
        
        tep_db_query("insert into view_counter_settings (page_name, view_active, view_spoofed, view_hackers, show_type, show_type_array, sort_by, sort_by_array, sort_direction_array, display_table, colors, group_data) values ('monitor', '" . tep_db_input($show_only_active) . "', '" . tep_db_input($show_only_spoofed) . "', '" . tep_db_input($show_only_hackers) . "', '" . tep_db_input($showType) . "', '" . $stringShow . "', '" . tep_db_input($sortBy) . "','" . $stringSort . "', '" . $sort_direction_array . "', '" . $display_table . "','" . $colors . "', '" . $group_data . "')");

    }        
}

/*******************************************************************************
Save the current settings - monitor only but can be shared
*******************************************************************************/
function SaveSettings($show_only_active, $show_only_spoofed, $show_only_hackers, $showType, $showTypeArray, $sortBy, $sortByArray, $sortDirectionArray, $display_table, $page_name = 'monitor') {
    $check_query = tep_db_query("select 1 from view_counter_settings");    
       
    $stringShow = base64_encode( serialize( $showTypeArray ) );  
    $stringSort = base64_encode( serialize( $sortByArray ) ); 
    $stringDirc = base64_encode( serialize( $sortDirectionArray ) ); 
    $showType   = tep_db_prepare_input($showType);    
    $group_data = '';
    
    if (tep_db_num_rows($check_query) > 0) {
        tep_db_query("update view_counter_settings set page_name = '" . tep_db_input($page_name) . "', view_active = '" . tep_db_input($show_only_active) . "', view_spoofed = '" . tep_db_input($show_only_spoofed) . "', view_hackers = '" . tep_db_input($show_only_hackers) . "', show_type = '" . tep_db_input($showType) . "', show_type_array = '" . $stringShow . "', sort_by = '" . tep_db_input($sortBy) . "', sort_by_array = '" . $stringSort . "', sort_direction_array = '" . $stringDirc . "', display_table = '" . tep_db_input($display_table) . "'"); 
    } else {    
        tep_db_query("insert into view_counter_settings (page_name, view_active, view_spoofed, view_hackers, show_type, show_type_array, sort_by, sort_by_array, sort_direction_array, display_table, colors, group_data) values ('monitor', '" . tep_db_input($show_only_active) . "', '" . tep_db_input($show_only_spoofed) . "', '" . tep_db_input($show_only_hackers) . "', '" . tep_db_input($showType) . "', '" . $stringShow . "', '" . tep_db_input($sortBy) . "','" . tep_db_input($stringSort) . "', '" . $stringDirc . "', '" . tep_db_input($display_table) . "','" . $colors . "', '" . $group_data . "')");
    }        
}

/*******************************************************************************
Save the current report settings
*******************************************************************************/
function SaveSettingsReports($page_name, $group_data) {
    $group_data = base64_encode( serialize( $group_data ) );  
    $check_query = tep_db_query("select 1 from view_counter_settings where page_name = '" . tep_db_input($page_name) . "'");    

    if (tep_db_num_rows($check_query) > 0) {
        tep_db_query("update view_counter_settings set group_data = '" . tep_db_input($group_data) . "' where page_name = '" . tep_db_input($page_name) . "'"); 
    } else {    
        tep_db_query("insert into view_counter_settings (page_name, group_data) values ('" . tep_db_input($page_name) . "', '" . tep_db_input($group_data) . "')");
    }        
}

/*******************************************************************************
Search a multi-array for the given needle
*******************************************************************************/
function VC_in_multi_array($needle, $haystack) {
    $in_multi_array = false;
    
    if (in_array($needle, $haystack)) {
        $in_multi_array = true;
    } else {
        foreach ($haystack as $key => $val) {
            if (is_array($val)) {
                if (VC_in_multi_array($needle, $val)) {
                    $in_multi_array = true;
                    break;
                }
            }
        }
    }
    return $in_multi_array;
}

  
/*******************************************************************************
Remove all apostrophes for use with ddrivetip code since they cause that code
to fail. Addslashes works in some cases but not all so it is best to just
remove them.
*******************************************************************************/
function RemoveApostrophes($string) {
   $string=str_replace('&#39;', '', $string);
   $string=str_replace("'", "", $string);
   $string=str_replace('"', '', $string);
   $string=preg_replace("/\\r\\n|\\n|\\r/", "<BR>", $string); 
   return $string;
}  

/*******************************************************************************
Show the customers account details if an account exists. If multiple accounts
exist, they are all show
*******************************************************************************/
function ShowAccountDetails($view, &$displayName, &$showPopup) {
    if (! tep_not_null($view['bot_name'])) {
        $displayName = TEXT_GUEST;
        return htmlspecialchars('<div style="clear:both;">' . TEXT_ACCOUNT_DETAILS_NOT_FOUND . '</div>');
    } else if ($view['isbot']) {
        $displayName = $view['bot_name'];
        $str = '<div class="accountDetail">' . $view['user_agent'] .'</div>';
        $showPopup = true;
        return htmlspecialchars($str);
    }
    
    $displayName = $view['bot_name']; 
 
    $db_query = tep_db_query("select * from customers c left join customers_info ci on c.customers_id = ci.customers_info_id where TRIM(Concat(customers_firstname, ' ', customers_lastname)) = '" . tep_db_input($view['bot_name']) . "'");
    if (tep_db_num_rows($db_query) > 0) {
        $str = '';
        while ($db = tep_db_fetch_array($db_query)) {
            $str .= '<div class="accountDetail accountDetailBorder">' . TEXT_ACCOUNT_DETAILS_EMAIL_ADDRESS . $db['customers_email_address'] .'</div>';
            $str .= '<div class="accountDetail">' . TEXT_ACCOUNT_DETAILS_PHONE . $db['customers_telephone'] .'</div>';
            $str .= '<div class="accountDetail">' . TEXT_ACCOUNT_DETAILS_CREATED_ON . $db['customers_info_date_account_created'] .'</div>';
            $str .= '<div class="accountDetail">' . TEXT_ACCOUNT_DETAILS_LAST_ACCESSED_ON . $db['customers_info_date_of_last_logon'] .'</div>';
            $str .= '<div class="accountDetail">' . TEXT_ACCOUNT_DETAILS_NUMBER_OF_LOGONS . $db['customers_info_number_of_logons'] .'</div>';
            
        }
        $showPopup = true;
        $str .= '<div class="accountDetailBorder"><div>';
        return htmlspecialchars($str);
    }    
    
    $showPopup = false;
    return '';
}

/*******************************************************************************
Show the contents of the cart, if active
*******************************************************************************/
function ShowCart($session_id, &$width = 300, $isadmin = '') {
    global $languages_id;

    if (tep_not_null($isadmin)) { 
        return TEXT_CART_ADMIN;        
    }

    if (! tep_not_null($session_id)) {
        return ERROR_SESSION_EMPTY;
    }
    
    $str = '';
    
    $cart_query = tep_db_query("select * from sessions where sesskey = '" . tep_db_input($session_id) . "'");
    if (tep_db_num_rows($cart_query)) {
        $cart = tep_db_fetch_array($cart_query);
        $cartArray = UnserializeSession(trim($cart['value']));
 
        if (count($cartArray['cart']->contents) > 0) {
            $langID = (isset($cartArray['languages_id']) && (int)$cartArray['languages_id'] > 0 ? $cartArray['languages_id'] : $languages_id);             
            $longest = 0;
            $str = '<div>';
            
            foreach ($cartArray['cart']->contents as $key => $data) {
                foreach ((array)$key as $id) {
                    if (strpos($id, '{') !== false) { //product has attributes
                        $idtmp = trim($id, '[ ]');
                        $pieces = explode('{', $idtmp);
                        $pName = tep_get_products_name($pieces[0], $langID);
                        $longest = GetLongestStr($pName, $longest);
                    
                        $str .= htmlspecialchars('<div class="cartProduct">ID: ' . $pieces[0] .'&nbsp;&nbsp;'. $cartArray['cart']->contents[$id]['qty'] .'&nbsp;x&nbsp;'. RemoveApostrophes($pName) . '</div>'); 
                        $items = count($pieces);
                        
                        for ($i = 1; $i < $items; ++$i) {
                            $attr = explode('}', $pieces[$i]);
                            $option_query = tep_db_query("select products_options_name from products_options where products_options_id = " . (int)$attr[0] . " and language_id = " . $langID); 
                            $option = tep_db_fetch_array($option_query);
                            $value_query = tep_db_query("select products_options_values_name from products_options_values where products_options_values_id = " . (int)$attr[1] . " and language_id = " . $langID); 
                            $value = tep_db_fetch_array($value_query);
                            $str .= htmlspecialchars('<div class="cartAttribute">&nbsp;&nbsp&nbsp;' . TEXT_CART_ATTR . '&nbsp;' . RemoveApostrophes($option['products_options_name']) . ' - '. RemoveApostrophes($value['products_options_values_name']) . '</div>');
                            $pName = TEXT_CART_ATTR . $option['products_options_name'] .$value['products_options_values_name'];
                            $longest = GetLongestStr($pName, $longest);
                        }  
                    } else {
                        $pName = tep_get_products_name($id, $langID);
                        $str .= htmlspecialchars('<div class="cartProduct">ID: ' . $id .'&nbsp;&nbsp;'. $cartArray['cart']->contents[$id]['qty'] .'&nbsp;x&nbsp;'. RemoveApostrophes($pName) . '</div>');
                        $longest = GetLongestStr($pName, $longest);
                    }
                    
                }
            }
            
            require_once('includes/classes/currencies.php');
            $currencies = new currencies();
        
            $str .= htmlspecialchars('<div class="cartTotal">' . TEXT_CART_TOTAL . ' ') .   $currencies->format($cartArray['cart']->total, true, $cartArray['currency']) .'</div>';
            $str .= '</div>';

            $width = ($longest + 6) * 9; //set the width of the box - add 6 characters for non-name text 7 multiply by 9 pixels/character
        } else {
            $str = TEXT_CART_EMPTY;
        }         
    } else {
        $str = TEXT_CART_SESSION_INACTIVE;    
    }
 
    return $str;
}

/*******************************************************************************
Show the details for this IP, if possible
*******************************************************************************/
function ShowIPDetails($record) {
    $str = '';
    
    if (is_array($record)) {
        foreach ($record as $key => $data) {         
            $str .= $key . ': ' . $data . '<br />';               
        } 
    } else {
        $str = TEXT_IP_DETAILS_NOT_FOUND;
    }

    return $str;
}

/*******************************************************************************
Show the name of the product, category, etc. in the listing when possible
*******************************************************************************/
function ShowPageItemName($arg) {
    $db_query = '';
    
    if (($pos = strpos($arg, 'cPath')) !== false) {
        $equal = strpos(substr($arg, $pos), '=');
        if ($equal !== false) {
            $id = (int)substr($arg, $equal + 1);
            $db_query = tep_db_query("select categories_name as name from categories_description where categories_id = " . (int)$id);
        }
        
    } elseif (($pos = strpos($arg, 'products_id')) !== false) {
        $equal = strpos(substr($arg, $pos), '=');
        if ($equal !== false) {
            $id = (int)substr($arg, $equal + 1);
            $db_query = tep_db_query("select products_name as name from products_description where products_id = " . (int)$id);
        }
    } elseif (($pos = strpos($arg, 'manufacturers_id')) !== false) {
        $equal = strpos(substr($arg, $pos), '=');
        if ($equal !== false) {
            $id = (int)substr($arg, $equal + 1);
            $db_query = tep_db_query("select manufacturers_name as name from manufacturers where manufacturers_id = " . (int)$id);
    }
    }    
    
    if (! empty($db_query) && tep_db_num_rows($db_query) > 0) {
        $db = tep_db_fetch_array($db_query);
        return '&nbsp;(' . (isset($db['name'][VIEW_COUNTER_ITEM_NAME_LENGTH]) ? substr($db['name'], 0, VIEW_COUNTER_ITEM_NAME_LENGTH) : $db['name']) .')';
    }

    return '';
}

function whois_query($domain) {
    global $whoisservers;
    // fix the domain name:
    $domain = strtolower(trim($domain));
    $domain = preg_replace('/^http:\/\//i', '', $domain);
    $domain = preg_replace('/^www\./i', '', $domain);
    $domain = explode('/', $domain);
    $domain = trim($domain[0]);
 
    // split the TLD from domain name
    $_domain = explode('.', $domain);
    $lst = count($_domain)-1;
    $ext = $_domain[$lst];
  
    if (!isset($servers[$ext])){
        die('Error: No matching nic server found!');
    }
 
    $nic_server = $servers[$ext];
 
    $output = '';
 
 
 echo 'nisc '.$nic_server.'<br>';
    // connect to whois server:
    if ($conn = fsockopen ($nic_server, 43, $errno, $errstr)) {
        fputs($conn, $domain."\r\n");
        while(!feof($conn)) {
            $output .= fgets($conn,128);
        }
        fclose($conn);
    } else { 
        die('Error: Could not connect to ' . $nic_server . '!'); 
    }
 
    return $output;
}

/*******************************************************************************
Get the whois content of an ip by selecting the correct server 
*******************************************************************************/ 
function GetWhoIS($ip, $just_cidr = false) {
    $w = GetWhoISFromServer('whois.iana.org' , $ip);

    preg_match("#whois:\s*([\w.]*)#si" , $w , $data);   

    if (! isset($data[1])) {  //failed all attemtps
        echo ERROR_FSOCK_CONNECTION_FAILED;
        return;
    } 

    $whois_server = $data[1];

    // now get actual whois data
    $ip = 'n + '.$ip; //needed for ARIN
    $whois_data = GetWhoISFromServer($whois_server , $ip);  

    if (! tep_not_null($whois_data)) {  //failed all attemtps
        echo ERROR_FSOCK_CONNECTION_FAILED;
        return;
    } 

    if ($just_cidr) {
        $cidr = null;
        $cidr_start = strpos($whois_data, 'CIDR');
        if ($cidr_start !== false) {
            $cidr_stop = strpos(substr($whois_data, $cidr_start), "NetName");
            $cidr = trim(substr($whois_data, ($cidr_start + 5), ($cidr_stop+ 3))); //retuns just the CIDR
            
            if (strpos($cidr, 'Net') !== false) {
                $parts = explode('Net', $cidr); //remove the next line
                $cidr = $parts[0];
            }
        } elseif (($range_start = strpos($whois_data, 'NetRange')) !== false) { //this shouldn't happen since cidr will be present - but it may fail with "Array" if the range returns multiple CIDR's
            $range = substr($whois_data, ($range_start + 9), 62); //skip netrange:
            $parts = explode('CIDR', $range);
            $cidr = ConvertRangeToCIDR($parts[0]);    
        }   
   
        $country = '??';
        if (($pos = strpos($whois_data, 'Country')) !== false) {
            $country = trim(substr($whois_data, ($pos + 8), 11)); 
        }      
        
        $t = [];
        $t[] = $cidr;
        $t[] = $country;        
        return $t;
    }

    return $whois_data;  
}  

/*******************************************************************************
 Get the whois result from a whois server  
*******************************************************************************/ 
function GetWhoISFromServer($server , $ip) {
    $data = '';   
    $server = trim($server); // Before connecting, check whether server alive or not

    if (!strlen($server))	{
        print "Blank string provided for server" . PHP_EOL;
        return '';
    }

    $f = @fsockopen($server, 43, $errno, $errstr, 30);  // Create the socket and connect

    if (!$f) {          
        return '';  
    }  

    if (!stream_set_timeout($f , 3)) {    // Set the timeout limit for read    
        die('Unable to set set_timeout'); //Did this solve the problem ?  
    } 
    
    if ($f) {  // Send the IP to the whois server	
        fputs($f, "$ip\r\n");  
    }

    /*
     Theory : stream_set_timeout must be set after a write and before a read for it to take effect on the read operation
     If it is set before the write then it will have no effect : http://in.php.net/stream_set_timeout
    */     
    if (!stream_set_timeout($f , 3)) {  //Set the timeout limit for read 
        die('Unable to stream_set_timeout');    #Did this solve the problem ?  
    }  

    //Set socket in non-blocking mode  
    stream_set_blocking ($f, 0 );  
     
    if ($f) {  //If connection still valid 
        while (!feof($f)) {  
            $data .= fread($f , 128);  
        }  
    }  

    return $data;  
} 
 
/*******************************************************************************
Used to unserialize a sesssion - normal unserialze won't work
*******************************************************************************/
if (! function_exists('UnserializeSession')) {
function UnserializeSession($session_data) {
     $return_data = [];
     $offset = 0;
     while ($offset < strlen($session_data)) {
         if (!strstr(substr($session_data, $offset), "|")) {
             throw new Exception("invalid data, remaining: " . substr($session_data, $offset));
         }
         $pos = strpos($session_data, "|", $offset);
         $num = $pos - $offset;
         $varname = substr($session_data, $offset, $num);
         $offset += $num + 1;
         $data = unserialize(substr($session_data, $offset));
         $return_data[$varname] = $data;
         $offset += strlen(serialize($data));
     }
     return $return_data;    
}
}

/*******************************************************************************
Sort a multi array - data field must be named as shown
*******************************************************************************/
function VC_SortMultiArray(&$array, $field = 'text') {
    array_multisort(array_map(function($element) {
      return $element['text'];
    }, $array), SORT_ASC, SORT_NUMERIC, $array);
}
function VC_SortMultiArrayOnCount(&$array) {
    array_multisort(array_map(function($element) {
      return $element['cnt'];
    }, $array), SORT_DESC, SORT_NUMERIC, $array);
}

function VC_SortMultiArray_ORIG($a, $b) {
    return strnatcasecmp($a["text"], $b["text"]);
}

function VC_SortMultiArrayOnCount_ORIG($a, $b) {
    if (! isset($a['count']) || ! isset($b['count'])) return 0;
    return $a['count'] < $b['cnt'];
};
 
/*******************************************************************************
Multi-select menu
*******************************************************************************/
function VCMultiSelectMenu($name, $values, $selected_vals, $params = '', $required = false) {
    $field = '<select name="' . $name . '"';
    if ($params) $field .= ' ' . $params;
    $field .= ' multiple="multiple">';
    
    for ($i=0; $i<sizeof($values); ++$i) {
        if ($values[$i]['id']) {
            $field .= '<option value="' . $values[$i]['id'] . '"';
            if ( ((strlen($values[$i]['id']) > 0) && ($name == $values[$i]['id'])) ) {
                $field .= '  selected="selected"';
            } elseif (tep_not_null($selected_vals)) {
                for ($j=0; $j<sizeof($selected_vals); ++$j) {
                    if ($selected_vals[$j]['id'] == $values[$i]['id']) {
                        $field .= ' selected="selected"';
                    }
                }
            }
        }

        $field .= '>' . $values[$i]['text'] . '</option>';
    }
    $field .= '</select>';

    if ($required) $field .= TEXT_FIELD_REQUIRED;

    return $field;
}

/*******************************************************************************
Write the altered .htaccess file
*******************************************************************************/
function WriteHTAccess($htaccessfile, $arrayWrite) {
    if (file_exists($htaccessfile)) {
        $fp = fopen($htaccessfile, "w" );
        if ($fp) {
            foreach ($arrayWrite as $line) {
                fputs($fp, $line);      
            }
            
            fclose($fp);
            return true;
        }
    }
    return false;
}